@@ -1,3 +1,4 @@
+from itertools import chain
 from typing import Iterable, TYPE_CHECKING
 
 from typing_extensions import Literal
@@ -11,8 +12,9 @@ from .style import StyleType
 if TYPE_CHECKING:
     from .console import Console, ConsoleOptions, RenderResult, RenderableType
 
-AlignValues = Literal["left", "center", "right"]
-AlignMethod = AlignValues  # TODO: deprecate AlignValues
+AlignMethod = Literal["left", "center", "right"]
+VerticalAlignMethod = Literal["top", "middle", "bottom"]
+AlignValues = AlignMethod  # TODO: deprecate AlignValues
 
 
 class Align(JupyterMixin):
@@ -22,8 +24,10 @@ class Align(JupyterMixin):
         renderable (RenderableType): A console renderable.
         align (AlignMethod): One of "left", "center", or "right""
         style (StyleType, optional): An optional style to apply to the background.
+        vertical (Optional[VerticalAlginMethod], optional): Optional vertical align, one of "top", "middle", or "bottom". Defaults to None.
         pad (bool, optional): Pad the right with spaces. Defaults to True.
         width (int, optional): Restrict contents to given width, or None to use default width. Defaults to None.
+        height (int, optional): Set height of align renderable, or None to fit to contents. Defaults to None.
 
     Raises:
         ValueError: if ``align`` is not one of the expected values.
@@ -32,21 +36,29 @@ class Align(JupyterMixin):
     def __init__(
         self,
         renderable: "RenderableType",
-        align: AlignMethod,
+        align: AlignMethod = "left",
         style: StyleType = None,
         *,
+        vertical: VerticalAlignMethod = None,
         pad: bool = True,
         width: int = None,
+        height: int = None,
     ) -> None:
         if align not in ("left", "center", "right"):
             raise ValueError(
-                f'invalid value for align, expected "left", "center", "right" (not {align!r})'
+                f'invalid value for align, expected "left", "center", or "right" (not {align!r})'
+            )
+        if vertical is not None and vertical not in ("top", "middle", "bottom"):
+            raise ValueError(
+                f'invalid value for vertical, expected "top", "middle", or "bottom" (not {vertical!r})'
             )
         self.renderable = renderable
         self.align = align
         self.style = style
+        self.vertical = vertical
         self.pad = pad
         self.width = width
+        self.height = height
 
     def __repr__(self) -> str:
         return f"Align({self.renderable!r}, {self.align!r})"
@@ -57,11 +69,21 @@ class Align(JupyterMixin):
         renderable: "RenderableType",
         style: StyleType = None,
         *,
+        vertical: VerticalAlignMethod = None,
         pad: bool = True,
         width: int = None,
+        height: int = None,
     ) -> "Align":
         """Align a renderable to the left."""
-        return cls(renderable, "left", style=style, pad=pad, width=width)
+        return cls(
+            renderable,
+            "left",
+            style=style,
+            vertical=vertical,
+            pad=pad,
+            width=width,
+            height=height,
+        )
 
     @classmethod
     def center(
@@ -69,11 +91,21 @@ class Align(JupyterMixin):
         renderable: "RenderableType",
         style: StyleType = None,
         *,
+        vertical: VerticalAlignMethod = None,
         pad: bool = True,
         width: int = None,
+        height: int = None,
     ) -> "Align":
         """Align a renderable to the center."""
-        return cls(renderable, "center", style=style, pad=pad, width=width)
+        return cls(
+            renderable,
+            "center",
+            style=style,
+            vertical=vertical,
+            pad=pad,
+            width=width,
+            height=height,
+        )
 
     @classmethod
     def right(
@@ -81,25 +113,32 @@ class Align(JupyterMixin):
         renderable: "RenderableType",
         style: StyleType = None,
         *,
+        vertical: VerticalAlignMethod = None,
         pad: bool = True,
         width: int = None,
+        height: int = None,
     ) -> "Align":
         """Align a renderable to the right."""
-        return cls(renderable, "right", style=style, pad=pad, width=width)
+        return cls(
+            renderable,
+            "right",
+            style=style,
+            vertical=vertical,
+            pad=pad,
+            width=width,
+            height=height,
+        )
 
     def __rich_console__(
         self, console: "Console", options: "ConsoleOptions"
     ) -> "RenderResult":
-
         align = self.align
-
         width = Measurement.get(console, self.renderable).maximum
-
         rendered = console.render(
             Constrain(
                 self.renderable, width if self.width is None else min(width, self.width)
             ),
-            options,
+            options.update(height=None),
         )
         lines = list(Segment.split_lines(rendered))
         width, height = Segment.get_shape(lines)
@@ -147,11 +186,40 @@ class Align(JupyterMixin):
                     yield from line
                     yield new_line
 
-        iter_segments = generate_segments()
+        blank_line = (
+            Segment(f"{' ' * (self.width or options.max_width)}\n", style)
+            if self.pad
+            else Segment("\n")
+        )
+
+        def blank_lines(count) -> Iterable[Segment]:
+            if count > 0:
+                for _ in range(count):
+                    yield blank_line
+
+        vertical_height = self.height or options.height
+        iter_segments: Iterable[Segment]
+        if self.vertical and vertical_height is not None:
+            if self.vertical == "top":
+                bottom_space = vertical_height - height
+                iter_segments = chain(generate_segments(), blank_lines(bottom_space))
+            elif self.vertical == "middle":
+                top_space = (vertical_height - height) // 2
+                bottom_space = vertical_height - top_space - height
+                iter_segments = chain(
+                    blank_lines(top_space),
+                    generate_segments(),
+                    blank_lines(bottom_space),
+                )
+            else:  #  self.vertical == "bottom":
+                top_space = vertical_height - height
+                iter_segments = chain(blank_lines(top_space), generate_segments())
+        else:
+            iter_segments = generate_segments()
         if self.style is not None:
             style = console.get_style(self.style)
             iter_segments = Segment.apply_style(iter_segments, style)
-        return iter_segments
+        yield from iter_segments
 
     def __rich_measure__(self, console: "Console", max_width: int) -> Measurement:
         measurement = Measurement.get(console, self.renderable, max_width)
@@ -161,6 +229,10 @@ class Align(JupyterMixin):
 class VerticalCenter(JupyterMixin):
     """Vertically aligns a renderable.
 
+    Warn:
+        This class is deprecated and may be removed in a future version. Use Align class with
+        `vertical="middle"`.
+
     Args:
         renderable (RenderableType): A renderable object.
     """
@@ -180,10 +252,12 @@ class VerticalCenter(JupyterMixin):
         self, console: "Console", options: "ConsoleOptions"
     ) -> "RenderResult":
         style = console.get_style(self.style) if self.style is not None else None
-        lines = console.render_lines(self.renderable, options, pad=False)
+        lines = console.render_lines(
+            self.renderable, options.update(height=None), pad=False
+        )
         width, _height = Segment.get_shape(lines)
         new_line = Segment.line()
-        height = console.size.height
+        height = options.height or options.size.height
         top_space = (height - len(lines)) // 2
         bottom_space = height - top_space - len(lines)
         blank_line = Segment(f"{' ' * width}", style)
@@ -214,17 +288,17 @@ if __name__ == "__main__":  # pragma: no cover
     highlighter = ReprHighlighter()
     console = Console()
 
-    panel = Align.center(
-        Panel(
-            RenderGroup(
-                Align(highlighter("align='left'"), "left"),
-                Align(highlighter("align='center'"), "center"),
-                Align(highlighter("align='right'"), "right"),
-            ),
-            width=60,
-            style="on dark_blue",
-            title="VerticalCenter",
+    panel = Panel(
+        RenderGroup(
+            Align.left(highlighter("align='left'")),
+            Align.center(highlighter("align='center'")),
+            Align.right(highlighter("align='right'")),
         ),
-        style="on red",
+        width=60,
+        style="on dark_blue",
+        title="Algin",
+    )
+
+    console.print(
+        Align.center(panel, vertical="middle", style="on red", height=console.height)
     )
-    console.print(VerticalCenter(panel, style="on red"))
